# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ssc-sensor-suid.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'ssc-sensor-suid.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ssc_common_pb2 as ssc__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15ssc-sensor-suid.proto\x1a\x10ssc-common.proto\"X\n\x0eSscSuidRequest\x12\x11\n\tdata_type\x18\x01 \x02(\t\x12\x16\n\x0e\x65nable_updates\x18\x02 \x01(\x08\x12\x1b\n\x13only_default_values\x18\x03 \x01(\x08\":\n\x0fSscSuidResponse\x12\x11\n\tdata_type\x18\x01 \x02(\t\x12\x14\n\x03uid\x18\x02 \x03(\x0b\x32\x07.SscUid')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ssc_sensor_suid_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SSCSUIDREQUEST']._serialized_start=43
  _globals['_SSCSUIDREQUEST']._serialized_end=131
  _globals['_SSCSUIDRESPONSE']._serialized_start=133
  _globals['_SSCSUIDRESPONSE']._serialized_end=191
# @@protoc_insertion_point(module_scope)
